# -*- coding: utf-8 -*-
"""Wing logics."""


def xwing(s, p, wings, wing, verbose):
    """X-wing familiy."""
    from kaidoku.calc import naklist
    from kaidoku.misc import cell
    message = ''
    logic = ['X-wing', 'Swordfish', 'Jellyfish'][wings - 2]
    for i in range(2):
        for n in range(9):
            if len(wing[i][n][0]) > 1:
                nk, c, position, f = naklist(wings, wing[i][n][1])
                if nk:
                    pos = []
                    for k in f:
                        po = wing[i][n][0][k]
                        for m in c:
                            if m in wing[i][n][1][k]:
                                if i == 0:
                                    pos.append(po * 9 + m)
                                else:
                                    pos.append(m * 9 + po)
                    for j in pos:
                        p[j][n] = 0
                        if p[j].count(1) == 1:
                            s[j] = p[j].index(1) + 1
                    if verbose > 1:
                        if verbose == 2:
                            message = logic + ' can be found.'
                        else:
                            message = logic + ' of ' + str(n + 1) + ' in '
                            for j in position:
                                message += ['R', 'C'][i] + \
                                    str(wing[i][n][0][j] + 1)
                                if j != position[len(position) - 1]:
                                    message += ', '
                            message += ' and '
                            for j in c:
                                message += ['C', 'R'][i] + str(j + 1)
                                if j != c[len(c) - 1]:
                                    message += ', '
                            message += ' removes ' + str(n + 1) + ' from '
                            for j in pos:
                                message += cell(j)
                                if s[j] > 0:
                                    message += ' (=' + str(s[j]) + ')'
                                if j != pos[len(pos) - 1]:
                                    message += ', '
                    return (s, p, message, True, False)
    return (s, p, message, False, False)


def xywing(s, p, b, pair, verbose):
    """XY-wing."""
    from kaidoku.chain import (findchain)
    from kaidoku.misc import (cell)
    message = ''
    if len(pair) < 3:
        return (s, p, message, False, False)
    for i in pair:
        chainx = findchain(pair, [i[0], i[1]], b)
        chainy = findchain(pair, [i[0], i[2]], b)
        if len(chainx) > 1 and len(chainy) > 1:
            for x in chainx:
                for y in chainy:
                    # if x[1] == y[1] and x[0] not in b[y[0]]:
                    if x[1] == y[1]:
                        for bb in b[x[0]]:
                            if bb in b[y[0]]:
                                if s[bb] == 0 and p[bb][x[1] - 1] == 1 and i not in p[bb]:
                                    if verbose > 1:
                                        if verbose == 2:
                                            message = 'XY-wing can be found.'
                                        else:
                                            message = 'XY-wing of ' + \
                                                cell(
                                                    i[0]) + ' (' + str(i[1]) + ',' + \
                                                str(i[2]) + '), '
                                            message += cell(x[0]) + ' (' + \
                                                str(i[1]) + ',' + \
                                                str(x[1]) + '), ' + \
                                                cell(y[0]) + ' (' + str(i[2]) + \
                                                ',' + str(y[1]) + ') '
                                            message += 'removes ' + \
                                                str(x[1]) + ' from ' + \
                                                cell(bb) + '.'
                                    p[bb][x[1] - 1] = 0
                                    if sum(p[bb]) == 1:
                                        s[bb] = p[bb].index(1) + 1
                                        if verbose > 2:
                                            message += '\nTherefore ' + \
                                                cell(bb) + ' = ' + \
                                                str(s[bb]) + '.'
                                    return (s, p, message, True, False)

    return (s, p, message, False, False)


def xyzwing(s, p, b, boxl, comb, pb, verbose):
    """XYZ-wing."""
    from kaidoku.misc import (cell)
    message = ''
    a = [0, 0, 0]
    for i in range(9):
        for x in comb[i]:
            if len(x) == 3:
                for y in comb[i]:
                    if len(y) == 2:
                        if y[0] in x and y[1] in x:
                            xa = boxl[i][comb[i].index(x)]
                            ya = boxl[i][comb[i].index(y)]
                            xp = xa - [0, 3, 6, 27, 30, 33, 54, 57, 60][i]
                            if abs(xa - ya) > 3:
                                for za in pb[i, xp // 9]:
                                    if s[za] == 0 and sum(p[za]) == 2:
                                        if p[za][x[0] - 1] + p[za][x[1] - 1] + p[za][x[2] - 1] == 2:
                                            if p[za][y[0] - 1] + p[za][y[1] - 1] == 1:
                                                if p[za][y[0] - 1] == 1:
                                                    Z = y[0]
                                                else:
                                                    Z = y[1]
                                                a[0] = xa + [1, -1, -2][xp % 3]
                                                a[1] = xa + [2, 1, -1][xp % 3]
                                                for j in [0, 1]:
                                                    if s[a[j]] == 0:
                                                        if p[a[j]][Z - 1] == 1:
                                                            z = ()
                                                            for k in range(9):
                                                                if p[za][k] == 1:
                                                                    z = z + \
                                                                        (k + 1,)
                                                            message = xyzmessage(
                                                                xa, x, ya, y, za, z, Z, a[j], verbose)
                                                            p[a[j]][Z - 1] = 0
                                                            if sum(p[a[j]]) == 1:
                                                                s[a[j]] = p[a[j]].index(
                                                                    1) + 1
                                                                if verbose > 2:
                                                                    message += '\nTherefore ' + \
                                                                        cell(a[j]) + ' = ' + \
                                                                        str(s[a[j]]
                                                                            ) + '.'
                                                            return (s, p, message, True, False)
                            if (xa - ya) % 3 > 0:
                                for za in pb[i, 3 + xp % 3]:
                                    if s[za] == 0 and sum(p[za]) == 2:
                                        if p[za][x[0] - 1] + p[za][x[1] - 1] + p[za][x[2] - 1] == 2:
                                            if p[za][y[0] - 1] + p[za][y[1] - 1] == 1:
                                                if p[za][y[0] - 1] == 1:
                                                    Z = y[0]
                                                else:
                                                    Z = y[1]
                                                a[0] = xa + \
                                                    [9, -9, -18][xp // 9]
                                                a[1] = xa + [18, 9, -9][xp // 9]
                                                for j in [0, 1]:
                                                    if s[a[j]] == 0:
                                                        if p[a[j]][Z - 1] == 1:
                                                            z = ()
                                                            for k in range(9):
                                                                if p[za][k] == 1:
                                                                    z = z + \
                                                                        (k + 1,)
                                                            message = xyzmessage(
                                                                xa, x, ya, y, za, z, Z, a[j], verbose)
                                                            p[a[j]][Z - 1] = 0
                                                            if sum(p[a[j]]) == 1:
                                                                s[a[j]] = p[a[j]].index(
                                                                    1) + 1
                                                                if verbose > 2:
                                                                    message += '\nTherefore ' + \
                                                                        cell(a[j]) + ' = ' + \
                                                                        str(s[a[j]]
                                                                            ) + '.'
                                                            return (s, p, message, True, False)
    return (s, p, message, False, False)


def xyzmessage(xa, x, ya, y, za, z, Z, a, verbose):
    from kaidoku.misc import (cell)
    message = ''
    if verbose > 1:
        if verbose == 2:
            message = 'XYZ-wing can be found.'
        else:
            message = 'XYZ-wing of ' + cell(xa) + ' ' + str(x) + ' ' + \
                cell(ya) + ' ' + str(y) + ' ' + cell(za) + ' ' + str(z) + ' ' + \
                'removes ' + str(Z) + ' from ' + cell(a) + '.'
    return message



# -*- coding: utf-8 -*-
"""Modules for generating sudoku figures."""
import os.path

from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont


def drawimage(s, p, lastmove, label, imgfile, figure, mark):
    """Draw sudoku image."""
    font = figure['font']
    textcolor = figure['color']
    lastcolor = figure['lastcolor']
    if mark:
        size = figure['markedsize']
    else:
        size = figure['normalsize']
    axis = figure['axis']
    if size in scalelist():
        scale = scalelist()[size]
    else:
        size = 'medium'
        scale = 1.5

    blankfile = os.path.abspath(
        os.path.dirname(__file__)) + '/data/blank-' + size
    if not os.path.isfile(blankfile + '.eps'):
        blankimage(blankfile, scale)
    c = Image.open(blankfile + '.eps')
    draw = ImageDraw.Draw(c)
    try:
        tfont = ImageFont.truetype(font, int(16 * scale))
    except Exception:
        print('Font file cannot be loaded: ', font)
        print('Rewrite font variable in the configuration file.')
        return True
    nfont = ImageFont.truetype(font, int(24 * scale))
    sfont = ImageFont.truetype(font, int(9 * scale))
    if axis == 'numeric':
        draw.text((20 * scale, 0), label, font=tfont, fill='#000')
    else:
        draw.text((20 * scale, 10 * scale), label, font=tfont, fill='#000')
    if axis == 'numeric':
        for i in range(9):
            draw.text((scale, (45 + i * 28.3) * scale), str(i + 1),
                      font=sfont, fill=textcolor)
        for j in range(9):
            draw.text(((24 + j * 28.3) * scale, 21 * scale), str(j + 1),
                      font=sfont, fill=textcolor)
    for i in range(9):
        for j in range(9):
            n = int(s[i * 9 + j])
            if n == 0 and mark:
                for m in range(9):
                    if p[i * 9 + j][m] == 1:
                        draw.text(((16 + j * 28.3 + (m % 3) * 8) * scale,
                                   (36 + i * 28.3 +
                                    (m // 3) * 8) * scale), str(m + 1),
                                  font=sfont, fill=textcolor)
            if n > 0:
                if i * 9 + j == lastmove:
                    color = lastcolor
                else:
                    color = textcolor
                draw.text(((20 + j * 28.3) * scale, (36 + i * 28.3)
                           * scale), str(n), font=nfont, fill=color)
    c.save(imgfile, 'JPEG', quality=100, optimize=True)
    return False


def scalelist():
    return {'small': 0.8, 'medium': 1, 'large': 1.5, 'x-large': 2}


def blankimages():
    """Draw blank images in all scales"""
    for size in scalelist():
        scale = scalelist()[size]
        blankfile = os.path.abspath(
            os.path.dirname(__file__)) + '/data/blank-' + size
        blankimage(blankfile, scale)
    return


def blankimage(file, scale):
    """Draw blank image."""
    import pyx
    m = 0.4  # margin
    tm = 1.2  # top margin
    c = pyx.canvas.canvas()
    c.stroke(pyx.path.rect(0, 0, (9 + 2 * m) * scale,
                           (9 + m + tm) * scale), [pyx.color.grey(1)])
    for i in range(10):
        if i % 3 == 0:
            c.stroke(pyx.path.line((i + m) * scale, m * scale,
                                   (i + m) * scale, (9 + m) * scale),
                     [pyx.style.linewidth.THIck])
            c.stroke(pyx.path.line(m * scale, (i + m) * scale, (9 + m)
                                   * scale, (i + m) * scale),
                     [pyx.style.linewidth.THIck])
        else:
            c.stroke(pyx.path.line((i + m) * scale, m *
                                   scale, (i + m) * scale, (9 + m) * scale))
            c.stroke(pyx.path.line(m * scale, (i + m) *
                                   scale, (9 + m) * scale, (i + m) * scale))
    c.writeEPSfile(file)
    return



# -*- coding: utf-8 -*-
"""Modules for commandline interpretation."""
import copy
import datetime
import os.path

from configobj import ConfigObj

from kaidoku.calc import possible
from kaidoku.calc import solve
from kaidoku.calc import solveone
from kaidoku.create import analyze
from kaidoku.create import append_database
from kaidoku.create import merge
from kaidoku.create import reanalyze
from kaidoku.create import reanalyze_giveup
from kaidoku.create import show_status
from kaidoku.help import advancedhelp
from kaidoku.help import helpmessage
from kaidoku.image import drawimage
from kaidoku.misc import blank
from kaidoku.misc import box
from kaidoku.misc import check
from kaidoku.misc import conv
from kaidoku.misc import current
from kaidoku.misc import duplicate
from kaidoku.misc import lev
from kaidoku.misc import openappend
from kaidoku.misc import pbox
from kaidoku.output import output
from kaidoku.output import short
from kaidoku.output import url


def command(arg, config):
    """Interpret command."""
    import shutil
    import webbrowser
    c = arg[0]
    if c == 'a' or c == 'ac':
        bookmark = config["bookmark"]
        move2 = config["move"]
        if c == 'a':
            config["move"] = []
        if len(arg) > 1:
            try:
                verbose = int(arg[1])
            except Exception:
                verbose = 1
        else:
            verbose = 1
        config, err = show(c, verbose, config)
        config["move"] = move2
        return config
    if c == 'b':
        move = config["move"]
        if move == []:
            print('Take back unavaillable because we are at initial position.')
            return config
        config["move"] = move[:len(move) - 1]
        config, err = show(c, 0, config)
        config = command(['c'], config)
        return config
    if c == 'c' or c == 'u' or c == 'jpg' or c == 'jm':
        config, err = show(c, 0, config)
        if err:
            print('Going back to problem No. 1.')
            level = int(config['level'])
            pointer = config['pointer']
            pointer[level] = 1
            config['pointer'] = pointer
            config, err = show(c, 0, config)
            if err:
                print(
                    'There is no problem in level ' + str(config['level']) +
                    '. Change level with l command.')
                return config
        return config
    if c == 'h':
        print(helpmessage())
        return config
    if c == 'ha':
        print(advancedhelp())
        return config
    if c == 'i' or c == 'ii' or c == 'iii':
        config, err = show(c, 0, config)
        return config
    if c == 'l':
        level = int(config["level"])
        try:
            le = int(arg[1])
        except Exception:
            return config
        if le > 0 and le < 10:
            level = le
            config['level'] = level
        config['move'] = []
        config = command(['c'], config)
        return config
    if c == 'n' or c == 'p' or c == 'j' or c == 'initial':
        pointer = config['pointer']
        level = int(config['level'])
        n = pointer[level]
        if int(config["level"]) == 0:
            if c == 'n' or c == 'p':
                return config
        else:
            n = int(n)
        n2 = n
        if c == 'p':
            n -= 1
            if n == 0:
                print('No previous problem.')
                n = 1
                return config
        if c == 'n':
            n += 1
        if c == 'j':
            if len(arg) < 2:
                return config
            try:
                n = int(arg[1])
            except Exception:
                return config
        pointer = config['pointer']
        pointer[level] = n
        config['pointer'] = pointer
        config['move'] = []
        config, err = show(c, 0, config)
        if err:
            pointer[level] = n2
            config['pointer'] = pointer
        return config
    if c.isdigit():  # move
        move = config["move"]
        row = int(c) // 100
        if row < 1 or row > 9:
            print('Invalid move. If you want to fill Row 3' +
                  'Column 5 with 7, type 357.')
            return config
        column = int(int(c) - row * 100) // 10
        num = int(c) % 10
        i = (row - 1) * 9 + column - 1
        move.append(int(c))
        config['move'] = move
        config, err = show(c, 0, config)
        return config
    if c == 'all':
        if len(arg) > 1:
            verbose = int(arg[1])
        else:
            verbose = 0
        file = os.path.expanduser(config["file"])
        level = int(config["level"])
        analyze(file, level, verbose)
        return config
    if c == 'append' or c == 'bp':
        file = os.path.expanduser(config["file"])
        maxtime = int(config['maxtime'])
        bookmark = config['bookmark']
        # If filename is written in bookmark, read bookmark from the file
        if type(config['bookmark']) is str:
            confbookmark = ConfigObj(
                os.path.expanduser(bookmark), encoding='utf-8')
            bookmark = confbookmark['bookmark']
        if len(arg) > 2:
            verbose = int(arg[2])
        else:
            verbose = 1
        if len(arg) == 1:
            print('Position not specified.')
            return(config)
        try:
            s, err = conv(arg[1])
        except Exception:
            print('Invalid position.')
            return config
        if err:
            print(s)
            return config
        s2 = copy.copy(s)
        s, message, level, solved, err = solve(s, 0, blank(s), maxtime)
        if solved:
            if err:
                print('This sudoku has multiple solutions.')
            else:
                if c == 'append':  # Append to book
                    infile = open(file)
                    for line in infile:
                        data = line.strip().split(' ')
                        s3 = conv(data[1])[0]
                        if s2 == s3:
                            print('This sudoku is already in the book.')
                            infile.close
                            return config
                    infile.close
                    out = openappend(file)
                    if output == 'error':
                        print('Unable to write a file:', file)
                        return config
                    out.write(str(level) + ' ' + short(s2) + '\n')
                    out.close
                    print(
                        'Valid sudoku. Appended to the book by level' +
                        +str(level))
                else:  # Add to bookmark
                    problem = short(s2)
                    comment = ''
                    if 'bookmark' in config:
                        bookmark = config['bookmark']
                        # If filename is written in bookmark, read bookmark from the file
                        if type(config['bookmark']) is str:
                            bookmark = ConfigObj(os.path.expanduser(
                                bookmark), encoding='utf-8')['bookmark']
                        num = 1
                        while 'b' + str(num) in bookmark:
                            num += 1
                        label = 'b' + str(num)
                        for lab in bookmark:
                            if bookmark[lab]['problem'] == problem:
                                label = lab
                                print('Already labeled as {0}'.format(label))
                                if 'comment' in bookmark[label]:
                                    comment = bookmark[label]['comment']
                                    print('Comment = {0}'.format(comment))
                                    break
                                else:
                                    bookmark = {}
                    try:
                        com = input('Comment on this position: ')
                    except Exception:
                        com = ''
                        print('')
                    if com != '':
                        comment = com
                    bookmark[label] = {}
                    bookmark[label]['problem'] = problem
                    bookmark[label]['move'] = ''
                    bookmark[label]['added'] = datetime.datetime.now(
                    ).strftime('%Y/%m/%d')
                    if comment != '':
                        bookmark[label]['comment'] = comment
                    if type(config['bookmark']) is str:
                        confbookmark['bookmark'] = bookmark
                        confbookmark.write()
                    else:
                        config['bookmark'] = bookmark
        else:
            if err:
                print('This sudoku has no solution.')
            else:
                print(
                    'This sudoku cannot be solved in ' +
                    str(maxtime) + ' seconds.')
        return config
    if c == 'book':
        file = os.path.expanduser(config["file"])
        show_status(file)
        return config
    if c == "config":
        ooc = ['bookmark', 'move', 'pointer', 'level']  # Out of control
        for i in config:
            if i not in ooc:
                print('{0} = {1}'.format(i, config[i]))
        while True:
            try:
                entry = input('Entry to edit: ')
            except Exception:
                print()
                return config
            if entry in config and entry not in ooc:
                if type(config[entry]) is str:
                    try:
                        value = input(entry + ' = ')
                    except Exception:
                        print()
                        return config
                    if entry == 'maxtime':
                        try:
                            value = int(value)
                        except Exception:
                            return config
                        if value == 0:
                            break
                        value = str(value)
                    config[entry] = value
                    for i in config:
                        if i not in ooc:
                            print('{0} = {1}'.format(i, config[i]))
                else:
                    print('Editing the config entry of ' + entry)
                    for i in config[entry]:
                        print('{0} = {1}'.format(i, config[entry][i]))
                    try:
                        subentry = input('Entry to edit in ' + entry + ': ')
                    except Exception:
                        print()
                        return config
                    if subentry in config[entry]:
                        if subentry[-4:] == 'size':
                            print('Available size: small, medium, large, x-large')
                        try:
                            value = input(subentry + ' = ')
                        except Exception:
                            print()
                            return config
                        if subentry[-4:] == 'size' and value not in ['small', 'medium', 'large', 'x-large']:
                            print('Size not available.')
                            return config
                        config[entry][subentry] = value
                        for i in config:
                            if i not in ooc:
                                print('{0} = {1}'.format(i, config[i]))
                    else:
                        return config
            else:
                if entry != '':
                    print('Entry not found.')
                return config
        return config
    if c == 'create':
        if len(arg) > 1:
            n = int(arg[1])
        else:
            n = 10
        file = os.path.expanduser(config["file"])
        datadir = os.path.expanduser(config["datadir"])
        giveup = os.path.expanduser(config["giveup"])
        creation = config["create"]
        if file == '':
            print('File not specified.')
            return config
        outfile = openappend(file)
        if outfile == 'error':
            print('Unable to write a file:', file)
            datadir = checkdatadir(datadir)
            config['datadir'] = datadir
            if os.path.isdir(datadir):
                file = datadir + '/sudoku.txt'
                print('Copying the system default file to ' + file)
                shutil.copyfile(os.path.abspath(
                    os.path.dirname(__file__)) + '/data/sudoku.txt', file)
                config["file"] = file
            else:
                return config
        else:
            outfile.close()
        print('Creating {0} new problems.'.format(n))
        append_database(file, datadir, giveup, n, creation)
        return config
    if c == 'ba':  # add bookmark
        file = os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        move = config["move"]
        bookmark = config["bookmark"]
        # If filename is written in bookmark, read bookmark from the file
        if type(bookmark) is str:
            confbookmark = ConfigObj(
                os.path.expanduser(bookmark), encoding='utf-8')
            bookmark = confbookmark['bookmark']
        n = pointer[level]
        if level == 0:  # bookmark
            if n not in bookmark:
                print('Label {0} not found in bookmark.')
                print('Going to level 3.')
                config['level'] = 3
                return config
            if 'problem' not in bookmark[n]:
                print('Label {0} broken.')
                print('Going to level 3.')
                config['level'] = 3
                return config
            problem = bookmark[n]['problem']
        else:
            n = int(n)
            infile = open(file, 'r')
            no = 0
            problem = ''
            for line in infile:
                data = line.strip().split(' ')
                if int(data[0]) == level:
                    no += 1
                    if no == n:
                        infile.close
                        problem = data[1]
                        break
            if problem == '':
                pointer[level] = 1
                print(
                    'Level {0} no. {1} not found. ' +
                    'Going back to no. 1'.format(level, n))
                return config
        s, err = conv(problem)
        if err:
            print(s)
            print('This problem is not valid.')
            return config
        s2 = copy.copy(s)
        if len(move) > 0:
            s2, move, message, err = current(s2, move)
            if err:
                config['move'] = move
                print(message)
                return config
        problem = short(s)
        mo = ''
        for i in move:
            mo += str(i)
        if len(arg) > 1:
            label = arg[1]
        else:
            label = 'b1'
        comment = ''
        if 'bookmark' in config:
            bookmark = config['bookmark']
            # If filename is written in bookmark, read bookmark from the file
            if type(bookmark) is str:
                confbookmark = ConfigObj(
                    os.path.expanduser(bookmark), encoding='utf-8')
                bookmark = confbookmark['bookmark']
            num = 1
            for i in bookmark:
                num += 1
            label = 'b' + str(num)
            for lab in bookmark:
                if bookmark[lab]['problem'] == problem and bookmark[lab]['move'] == mo:
                    label = lab
                    print('Already labeled as {0}'.format(label))
                    if 'comment' in bookmark[label]:
                        comment = bookmark[label]['comment']
                        print('Comment = {0}'.format(comment))
                    break
        else:
            bookmark = {}
        try:
            com = input('Comment on this position: ')
        except Exception:
            com = ''
            print('')
        if com != '':
            comment = com
        bookmark[label] = {}
        bookmark[label]['problem'] = problem
        bookmark[label]['move'] = mo
        bookmark[label]['added'] = datetime.datetime.now().strftime('%Y/%m/%d')
        if comment != '':
            bookmark[label]['comment'] = comment
        if type(config['bookmark']) is str:
            confbookmark['bookmark'] = bookmark
            confbookmark.write()
        else:
            config['bookmark'] = bookmark
        return config
    if c == 'bl':
        if 'bookmark' in config:
            bookmark = config['bookmark']
            # If filename is written in bookmark, read bookmark from the file
            if type(bookmark) is str:
                bookmark = ConfigObj(os.path.expanduser(
                    bookmark), encoding='utf-8')['bookmark']
        else:
            print('No bookmark.')
            return config
        for label in bookmark:
            if 'comment' in bookmark[label]:
                comment = bookmark[label]['comment']
            else:
                comment = ''
            if 'added' in bookmark[label]:
                added = bookmark[label]['added']
            else:
                added = ' ' * 10
            print('{0}   {1}   {2}'.format(added, label, comment))
        return config
    if c == 'br':
        if 'bookmark' in config:
            bookmark = config['bookmark']
            # If filename is written in bookmark, read bookmark from the file
            if type(bookmark) is str:
                confbookmark = ConfigObj(
                    os.path.expanduser(bookmark), encoding='utf-8')
                bookmark = confbookmark['bookmark']
        else:
            print('No bookmark.')
            return config
        if len(arg) == 1:
            print('Label is not specified.')
            return config
        n = arg[1]
        level = 0
        if n not in bookmark:
            print('Label {0} not found in bookmark.'.format(n))
            print('Going to level 3.')
            config['level'] = 3
            return config
        mo = bookmark[n]['move']
        move = []
        while len(mo) > 2:
            move.append(int(mo[:3]))
            mo = mo[3:]
        config['level'] = 0
        pointer = config['pointer']
        pointer[0] = n
        config['pointer'] = pointer
        if type(config['bookmark']) is str:
            confbookmark['bookmark'] = bookmark
            confbookmark.write()
        else:
            config['bookmark'] = bookmark
        config['move'] = move
        config, err = show(c, 0, config)
        return config
    if c == 'giveup':
        if len(arg) > 1:
            t = int(arg[1])
        else:
            t = 10
        giveup = os.path.expanduser(config["giveup"])
        reanalyze_giveup(giveup, t)
        return config
    if c == 'html':
        datadir = config['datadir']
        html = os.path.join(datadir, 'current.html')
        here = os.path.abspath(os.path.dirname(__file__))
        shutil.copyfile(os.path.join(here, 'data/current.html'), html)
        webbrowser.open('file://' + html)
        return config
    if c == 'import':
        file = os.path.expanduser(config["file"])
        file2 = os.path.expanduser(config["file2"])
        merge(file, file2)
        return config
    if c == 'reanalyze':
        file = os.path.expanduser(config["file"])
        file2 = os.path.expanduser(config["file2"])
        reanalyze(file, file2)
        return config
    if c in ['solve', 'check']:
        maxtime = int(config['maxtime'])
        if len(arg) > 2:
            verbose = int(arg[2])
        else:
            verbose = 1
        if len(arg) == 1:
            print('Position not specified.')
            return(config)
        try:
            s, err = conv(arg[1])
        except Exception:
            print('Invalid position.')
            return config
        if err:
            print(s)
            return config
        print(output(s))
        message, err = check(s)
        if err:
            print(message)
            return config
        if blank(s) == 0:
            print('Already solved.')
            return config
        if c == 'solve':
            solveprint(s, verbose, blank(s), maxtime)
        else:
            s2 = copy.copy(s)
            s2, message, level2, solved, err = solve(s2, 0, blank(s), maxtime)
            if err:
                if solved:
                    print('This position has multiple solutions.')
                else:
                    print(
                        'There is no solution for this position. ')
                return config
            if verbose < 2:
                print('This position has a unique solution.')
                return config
            p = possible(s)
            b = box()
            pb = pbox()
            blank1 = blank(s)
            start = datetime.datetime.now()
            endtime = start + datetime.timedelta(seconds=maxtime)
            s2 = copy.copy(s)
            s2, p, message, logic, depth, found, err = solveone(
                s2, p, 4, 0, blank1, endtime, b, pb)
            if logic == 'Naked single' or logic == 'Hidden single':
                if logic == 'Naked single':
                    print('Look at {0}. What number is available?'.format(
                        message[14:18]))
                else:
                    print(message[:message.index(':')] + 'can be found.')
                return config
            if verbose < 3:
                print('Think candidates of the cells.')
                return config
            logi = [logic]
            mes = [message]
            while blank(s2) == blank1:
                s2, p, message, logic, depth, found, err = solveone(
                    s2, p, 4, 0, blank1, endtime, b, pb)
                logi.append(logic)
                mes.append(message)
            if verbose == 3:
                if len(logi) > 1:
                    print('Following logics are successively used.')
                for i in range(len(logi)):
                    print(logi[i])
            else:
                for message in mes:
                    print(message)
        return config
    if c == 'sp':
        config, err = show(c, 0, config)
        return config

    if c == 'test':
        from kaidoku.test import test_all
        test_all()
        return config
    if c == 'blankimages':
        from kaidoku.image import blankimages
        blankimages()
        return config

    print('Invalid command. Type h for help.')
    return config


def show(c, verbose, config):
    """Show current position."""
    file = os.path.expanduser(config["file"])
    move = config["move"]
    level = int(config["level"])
    pointer = config['pointer']
    maxtime = int(config['maxtime'])
    bookmark = config["bookmark"]
    # If filename is written in bookmark, read bookmark from the file
    if type(config['bookmark']) is str:
        bookmark = ConfigObj(os.path.expanduser(bookmark),
                             encoding='utf-8')['bookmark']
    n = pointer[level]
    datadir = config["datadir"]
    infile = open(file, 'r')
    no = 0
    if level == 0:  # bookmark
        if n not in bookmark:
            print('Label {0} not found in bookmark.'.format(n))
            return config, False
        if 'problem' not in bookmark[n]:
            print('Label {0} broken.'.format(n))
            return config, False
        problem = bookmark[n]['problem']
    else:
        n = int(n)
        infile = open(file, 'r')
        no = 0
        problem = ''
        for line in infile:
            data = line.strip().split(' ')
            if int(data[0]) == level:
                no += 1
                if no == n:
                    infile.close
                    problem = data[1]
                    break
        infile.close
        if problem == '':
            print('Level {0} no. {1} not found.'.format(level, n))
            return config, True  # not found

    s, err = conv(problem)

    if err:
        print(s)
        print('This problem is not valid.')
        return config, True

    if level == 0:
        if 'comment' in bookmark[n]:
            label = bookmark[n]['comment']
        else:
            label = str(n)
    else:
        label = 'Level ' + str(level) + ' No. ' + str(n)
    if len(move) > 0:
        s, move, message, err = current(s, move)
        if err:
            config['move'] = move
            print(message)
        if blank(s) == 0:
            label += ': solution'
        else:
            label += ': move ' + str(len(move))
    if c == 'c' or c == 'n' or c == 'p' or c == 'j' or c == 'initial' or c.isdigit():
        print(label)
        print(output(s))
        if blank(s) == 0:
            print('Now this problem is solved !')
        else:
            print('Type 3 digits (row, column, number) to put a number. i for hint.')
        if datadir != '':
            datadir = checkdatadir(datadir)
            config['datadir'] = datadir
            move = config['move']
            if len(move) > 0:
                lastmove = int(move[len(move) - 1]) // 10
                lastmove = (lastmove // 10 - 1) * 9 + lastmove % 10 - 1
            else:
                lastmove = 100
            imgfile = datadir + '/current.jpg'
            figure = config['figure']
            err = drawimage(s, '', lastmove, label, imgfile,
                            figure, False)
            if not err:
                print('See image by "html".')

    if c == 'a' or c == 'ac':
        print('\n' + label)
        solveprint(s, verbose, blank(s), maxtime)
    if c == 'u':
        print(url(s))
    if c == 'jpg' or c == 'jm':  # jpg
        p = possible(s)
        datadir = checkdatadir(datadir)
        config['datadir'] = datadir
        imgfile = datadir + '/current.jpg'
        if c == 'jpg':
            mark = False
        if c == 'jm':
            mark = True
        move = config['move']
        if len(move) > 0:
            lastmove = int(move[len(move) - 1]) // 10
            lastmove = (lastmove // 10 - 1) * 9 + lastmove % 10 - 1
        else:
            lastmove = 100
        figure = config['figure']
        err = drawimage(s, p, lastmove, label, imgfile, figure, mark)
        if not err:
            print('See image by "html".')
    if c == 'i' or c == 'ii' or c == 'iii' or c == 'sp':  # prepare solving
        if blank(s) == 0:
            print('Already solved.')
            return config, True
        s2 = copy.copy(s)
        s2, message, level2, solved, err = solve(s2, 0, blank(s), maxtime)
        if err:
            if solved:
                print('This position has multiple solutions.')
            else:
                print(
                    'There is no solution for this position. '
                    + 'You can take back one move with b.')
            return config, True
        p = possible(s)
        b = box()
        pb = pbox()
        blank1 = blank(s)
        start = datetime.datetime.now()
        endtime = start + datetime.timedelta(seconds=maxtime)
    if c == 'i' or c == 'ii' or c == 'iii':  # show hint
        s2 = copy.copy(s)
        s2, p, message, logic, depth, found, err = solveone(
            s2, p, 4, 0, blank1, endtime, b, pb)
        if logic == 'Naked single' or logic == 'Hidden single':
            if logic == 'Naked single':
                print('Look at {0}. What number is available?'.format(
                    message[14:18]))
            else:
                print(message[:message.index(':')] + 'can be found.')
        else:
            if c == 'i':
                print('Think candidates of the cells.')
                if datadir == '':
                    print('Use jm command to see the diagram of candidates.')
                else:
                    datadir = checkdatadir(datadir)
                    config['datadir'] = datadir
                    imgfile = datadir + '/current.jpg'
                    p = possible(s)
                    move = config['move']
                    if len(move) > 0:
                        lastmove = int(move[len(move) - 1]) // 10
                        lastmove = (lastmove // 10 - 1) * 9 + lastmove % 10 - 1
                    else:
                        lastmove = 100
                    figure = config['figure']
                    err = drawimage(s, p, lastmove, label, imgfile,
                                    figure, True)
                    if not err:
                        print('See image by "html".')
                print('For more hints, type ii.')
            if c == 'ii' or c == 'iii':
                logi = [logic]
                mes = [message]
                while blank(s2) == blank1:
                    s2, p, message, logic, depth, found, err = solveone(
                        s2, p, 4, 0, blank1, endtime, b, pb)
                    logi.append(logic)
                    mes.append(message)
                if c == 'ii':
                    if len(logi) > 1:
                        print('Following logics are successively used.')
                    for i in range(len(logi)):
                        print(logi[i])
                    print('See full explanation by typing iii.')
                else:
                    for message in mes:
                        print(message)
    if c == 'sp':
        logic = 'Naked single'
        while (logic == 'Naked single' or logic == 'Hidden single') and blank(s) > 0:
            s2 = copy.copy(s)
            s, p, message, logic, depth, found, err = solveone(
                s, p, 4, 0, blank(s), endtime, b, pb)
            if logic == 'Naked single' or logic == 'Hidden single':
                print(message)
                for i in range(81):
                    if s[i] != s2[i]:
                        j = (i // 9) * 10 + i % 9 + 11
                        m = j * 10 + s[i]
                        move.append(m)
            else:
                s = s2
        config, err = show('c', 0, config)
    return config, False


def checkdatadir(datadir):
    """Check data directory."""
    if datadir == '':
        print('Data directory does not exist.')
        while True:
            datadir = input(
                'Input name of the data directory (default: ~/kaidoku): ')
            if datadir == '':
                datadir = os.path.expanduser('~/kaidoku')
            datadir = os.path.expanduser(datadir)
            if not os.path.isdir(datadir):
                try:
                    os.mkdir(datadir)
                    break
                except Exception:
                    print('Error: directory cannot be created.', datadir)
                datadir = ''
            else:
                break
    return datadir


def solveprint(s, verbose, maxdepth, maxtime):
    """Analyze a problem (from a command through show)."""
    s, message, level, solved, err = solve(s, verbose, maxdepth, maxtime)
    if err:
        if verbose > 0:
            print(message)
        if solved:
            if verbose > 0:
                print("Invalid sudoku with multiple solutions.")
            if verbose > 4:
                print('Solution 1.')
                print(output(s[0]))
                print('Solution 2.')
                print(output(s[1]))
                s = duplicate(s[0], s[1])
                print('Common numbers in the 2 solutions')
                print(output(s))
        else:
            if verbose > 0:
                print("Invalid sudoku with no solution.")
    if solved:
        if verbose > 0:
            print('Valid sudoku with unique solution of level {0} ({1}).'.format(
                level, lev(level)))
        if verbose == 4:
            print('Here is the solution.')
            print(output(s))
    else:
        print('\nGive up with {0} blank cells.'.format(blank(s)))
        print(output(s))

    return



# -*- coding: utf-8 -*-
"""Calculation modules.

Modules for solving sudoku problems.
"""
import datetime
import itertools

from kaidoku.misc import blank
from kaidoku.misc import cell
from kaidoku.misc import check
from kaidoku.output import output


def solve(s, verbose, maxdepth, maxtime):
    """Solve a problem."""
    from kaidoku.misc import box
    from kaidoku.misc import pbox
    solved = False
    p = possible(s)
    b = box()
    pb = pbox()
    start = datetime.datetime.now()
    endtime = start + datetime.timedelta(seconds=maxtime)
    LevelPoint = int((blank(s) / 15.0)**3)  # 30 -> 8, 64 -> 77
    while (not solved):
        bl = blank(s)
        s, p, message, logic, depth, found, err = solveone(
            s, p, verbose, 0, maxdepth, endtime, b, pb)
        if err:
            if found:
                return (s, message, 0, True, True)
            else:
                return (s, message, 0, False, True)
        if found:
            if message != '':
                print(message)
            if verbose > 4 and blank(s) < bl:
                print(output(s))
            LevelPoint += {
                'Naked single': 0,
                'Hidden single':  3,
                'Pointing pair': 25,
                'Pointing triple': 25,
                'Naked pair': 25,
                'Naked triple': 40,
                'Hidden pair': 50,
                'Hidden triple': 70,
                'X-wing': 70,
                'XY-wing': 70,
                'XYZ-wing': 100,
                'Remote pairs': 100,
                'Naked quad': 120,
                'Hidden quad': 150,
                'Swordfish': 150,
                'Jellyfish': 200,
                'Chain of pairs': 200,
                'Chain of pairs (long)': 300,
                'Trial': 500,
                'Trial (deep)': 1000,
                'Search': 1500,
                'Deep search': 2500
            }[logic]
        else:
            return (s, 'Give up', 0, False, False)
        if blank(s) == 0:
            solved = True
        else:
            solved = False
    for i in range(9):
        if [0, 8, 45, 70, 120, 320, 1000, 2000, 5500][i] <= LevelPoint:
            level = i + 1
    return (s, 'Solved', level, True, False)


def solveone(s, p, verbose, depth, maxdepth, endtime, b, pb):
    """Solve one move."""
    from kaidoku.chain import pairchain
    from kaidoku.chain import remotepair
    from kaidoku.misc import boxlist
    from kaidoku.misc import combmir
    from kaidoku.misc import line
    from kaidoku.misc import pairs
    from kaidoku.misc import wingpos
    from kaidoku.search import search
    from kaidoku.search import trial
    from kaidoku.wing import xwing
    from kaidoku.wing import xywing
    from kaidoku.wing import xyzwing
    # Check validity
    message, err = check(s)
    if verbose == 0:
        message = ''
    if err:
        return (s, p, message, 0, depth, False, True)

    # Basic logic
    boxl = boxlist(s)
    s, p, message, found, err = naksing(s, p, b, verbose)  # Naked single
    if found or err:
        return (s, p, message, 'Naked single', depth, found, err)
    linescan = line()
    s, p, message, found, err = hidsing(
        s, p, linescan, verbose)  # Hidden single
    if found or err:
        return (s, p, message, 'Hidden single', depth, found, err)

    # Advanced logic

    # Pointing pair or triple
    s, p, logic, message, found, err = pointing(
        s, p, pb, verbose)
    if found or err:
        return (s, p, message, logic, depth, found, err)

    # Naked pair and triple
    comb, mirror = combmir(p, boxl)
    if depth < 2:
        s, p, message, found, err = naked(
            s, p, 2, boxl, comb, verbose)  # Naked pair
        if found or err:
            return (s, p, message, 'Naked pair', depth, found, err)
    if depth < 1:
        s, p, message, found, err = naked(
            s, p, 3, boxl, comb, verbose)  # Naked triple
        if found or err:
            return (s, p, message, 'Naked triple', depth, found, err)

    # Hidden pair and triple
    if depth < 2:
        s, p, message, found, err = hidden(
            s, p, 2, boxl, mirror, verbose)  # Hidden pair
        if found or err:
            return (s, p, message, 'Hidden pair', depth, found, err)
    if depth < 1:
        s, p, message, found, err = hidden(
            s, p, 3, boxl, mirror, verbose)  # Hidden triple
        if found or err:
            return (s, p, message, 'Hidden triple', depth, found, err)

    # Wing families
    wing = wingpos(boxl, mirror)
    pair, paircomb, pairdict = pairs(s, p)
    if depth < 2:
        s, p, message, found, err = xwing(
            s, p, 2, wing, verbose)  # X-wing
        if found or err:
            return (s, p, message, 'X-wing', depth, found, err)
        s, p, message, found, err = xywing(s, p, b, pair, verbose)  # XY-wing
        if found or err:
            return (s, p, message, 'XY-wing', depth, found, err)
        s, p, message, found, err = xyzwing(
            s, p, b, boxl, comb, pb, verbose)  # XYZ-wing
        if found or err:
            return (s, p, message, 'XYZ-wing', depth, found, err)

    if depth == 0:
        # Remote pairs
        if len(pair) > 3:
            s, p, message, found, err = remotepair(
                s, p, b, pair, pairdict, verbose)  # Remote pairs
            if found or err:
                return (s, p, message, 'Remote pairs', depth, found, err)

        # Naked and hidden quad
        s, p, message, found, err = naked(
            s, p, 4, boxl, comb, verbose)  # Naked quad
        if found or err:
            return (s, p, message, 'Naked quad', depth, found, err)

        s, p, message, found, err = hidden(
            s, p, 4, boxl, mirror, verbose)  # Hidden quad
        if found or err:
            return (s, p, message, 'Hidden quad', depth, found, err)

        # Swordfish and Jellyfish
        s, p, message, found, err = xwing(
            s, p, 3, wing, verbose)  # Swordfish
        if found or err:
            return (s, p, message, 'Swordfish', depth, found, err)
        s, p, message, found, err = xwing(
            s, p, 4, wing, verbose)  # Jellyfish
        if found or err:
            return (s, p, message, 'Jellyfish', depth, found, err)

        # Chain of pairs
        if len(pair) > 3:
            s, p, message, chainlength, found, err = pairchain(
                s, p, b, pair, paircomb, verbose)  # Chain of pairs
            if chainlength < 8:
                logic = 'Chain of pairs'  # Short chain
            else:
                logic = 'Chain of pairs (long)'  # Long chain
            if found or err:
                return (s, p, message, logic, depth, found, err)

    # Trial
    # Maximum steps of scan distinguished from full search of 1 depth
    maxstep = 15
    if depth == 0 and maxdepth < 999:
        s, p, step, message, found, err = trial(
            s, p, linescan, maxstep, verbose, b, pair)
        if step < 10:
            logic = 'Trial'
        else:
            logic = 'Trial (deep)'
        if found or err:
            return (s, p, message, logic, depth, found, err)

    # Search from minimum depth
    if maxdepth == 999:
        bla = blank(s)  # Creating mode
    else:
        bla = 0

    if depth == 0:
        if maxdepth > 2 and bla < 60:
            s, p, message, logic, depth2, found, err = search(
                s, p, verbose, 1, 2, endtime, b, pb)
            if found or err:
                return (s, p, message, logic, depth2, found, err)
        if maxdepth > 3 and bla < 45:
            s, p, message, logic, depth2, found, err = search(
                s, p, verbose, 1, 3, endtime, b, pb)
            if found or err:
                return (s, p, message, logic, depth2, found, err)
        if maxdepth > 4 and bla < 35:
            s, p, message, logic, depth2, found, err = search(
                s, p, verbose, 1, 4, endtime, b, pb)
            if found or err:
                return (s, p, message, logic, depth2, found, err)

    # Full search
    if maxdepth > depth:
        s, p, message, logic, depth, found, err = search(
            s, p, verbose, depth + 1, maxdepth, endtime, b, pb)
        if found or err:
            return (s, p, message, logic, depth, found, err)

    return (s, p, '', 0, depth, False, False)


def possible(s):
    """Make possibility array."""
    from kaidoku.misc import box
    p = [[]] * 81
    b = box()
    for i in range(81):
        if s[i] == 0:
            p[i] = [1] * 9
            for i2 in b[i]:
                if s[i2] > 0:
                    p[i][s[i2] - 1] = 0
        else:
            p[i] = [0] * 9
            p[i][s[i] - 1] = 1
    return (p)


def naksing(s, p, b, verbose):
    """Naked single."""
    for i in range(81):
        if s[i] == 0:
            for i2 in b[i]:
                v = s[i2]
                if v > 0:
                    p[i][v - 1] = 0
        else:
            p[i] = [10]

    found = False
    message = ''
    for i in range(81):
        if (sum(p[i])) == 0:
            if verbose > 0:
                message = 'This sudoku has no solution because ' + \
                    cell(i) + ' has no candidate.'
            return (s, p, message, False, True)
        if (sum(p[i])) == 1:
            i2 = p[i].index(1)
            found = True
            s[i] = i2 + 1
            if verbose > 3:
                if message == '':
                    message = ('Naked single: ')
                else:
                    message = message + ", "
                message = message + cell(i) + " = " + str(i2 + 1)

    if blank(s) == 0:
        message2, err = check(s)
        if verbose == 0:
            message2 = ''
        if err:
            return (s, p, message2, False, True)

    return (s, p, message, found, False)


def hidsing(s, p, linescan, verbose):
    """Hidden single."""

    for sc in linescan:
        for n in range(9):
            sum = 0
            for i in sc:
                if s[i] == 0:
                    sum = sum + p[i][n]
            if sum == 1:
                for i in sc:
                    if (s[i] == 0) and (p[i][n]) == 1:
                        s[i] = n + 1
                        if verbose < 4:
                            message = ''
                        else:
                            m = linescan.index(sc)
                            message = 'Hidden single in ' + ['box ', 'row ', 'column '][m // 9] + \
                                str(m % 9 + 1) + ' : ' + \
                                cell(i) + ' = ' + str(n + 1)
                        return (s, p, message, True, False)

    return (s, p, '', False, False)


def pointing(s, p, pb, verbose):
    """Pointing pair or triple."""
    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        blank = set()
        for i in range(9):
            if s[begin + (0, 1, 2, 9, 10, 11, 18, 19, 20)[i]] == 0:
                blank.add(i)
        for i in range(6):
            j = [set([0, 1, 2]), set([3, 4, 5]), set([6, 7, 8]),
                 set([0, 3, 6]), set([1, 4, 7]), set([2, 5, 8])][i]
            b2 = blank & j
            if len(b2) > 1 > 0:
                for n in range(9):
                    sum = 0
                    for a in b2:
                        sum = sum + \
                            p[begin + (0, 1, 2, 9, 10, 11, 18, 19, 20)[a]][n]
                    if sum > 1:
                        sum2 = 0
                        sum3 = 0
                        for a in (blank - b2):
                            sum2 = sum2 + \
                                p[begin + (0, 1, 2, 9, 10, 11,
                                           18, 19, 20)[a]][n]
                        for a in pb[(b, i)]:
                            if s[a] == 0:
                                sum3 = sum3 + p[a][n]
                        if (sum2 == 0) != (sum3 == 0):
                            logic = 'Pointing ' + \
                                ['', '', 'pair', 'triple'][sum]
                            message = ''
                            if verbose > 2:
                                message = logic + ' in box ' + str(b + 1)
                            if verbose > 3:
                                message = message + " removed " + \
                                    str(n + 1) + " from "
                            if (sum2 == 0):
                                for a in pb[(b, i)]:
                                    if s[a] == 0 and p[a][n] == 1:
                                        p[a][n] = 0
                                        if verbose > 3:
                                            message = message + cell(a) + ' '
                                        if p[a].count(1) == 1:
                                            if verbose > 3:
                                                message = message + \
                                                    '(=' + \
                                                    str(p[a].index(
                                                        1) + 1) + ') '
                                            s[a] = p[a].index(1) + 1
                            else:
                                for a in (blank - b2):
                                    a2 = begin + (0, 1, 2, 9, 10,
                                                  11, 18, 19, 20)[a]
                                    if p[a2][n] == 1:
                                        r = p[a2]
                                        r[n] = 0
                                        p[a2] = r
                                        if verbose > 3:
                                            message = message + cell(a2) + ' '
                                        if p[a2].count(1) == 1:
                                            if verbose > 3:
                                                message = message + \
                                                    '(=' + \
                                                    str(p[a2].index(
                                                        1) + 1) + ') '
                                            s[a2] = p[a2].index(1) + 1
                            return (s, p, logic, message, True, False)
    return (s, p, '', '', False, False)


def naked(s, p, length, boxl, comb, verbose):
    """Naked pair, triple or quad."""
    message = ''
    for i in range(len(comb)):
        list = comb[i]
        if len(list) >= length * 2:
            nk, c, position, f = naklist(length, list)
            if nk:
                if verbose > 1:
                    message = 'Naked ' + ['single', 'pair', 'triple', 'quad'][length - 1] + ' in ' + \
                        ['box ', 'row ', 'column '][i // 9] + \
                        str(i % 9 + 1)
                if verbose > 2:
                    message = message + ' made removal from'
                for j in f:
                    k = boxl[i][j]
                    for m in range(length):
                        p[k][c[m] - 1] = 0
                    if verbose > 2:
                        message = message + ' ' + cell(k)
                    if p[k].count(1) == 1:
                        if verbose > 2:
                            message = message + \
                                '(=' + str(p[k].index(1) + 1) + ') '
                        s[k] = p[k].index(1) + 1
                return (s, p, message, True, False)
    return (s, p, '', False, False)


def hidden(s, p, length, boxl, mirror, verbose):
    """Hidden pair, triple or quad."""
    message = ''
    for i in range(len(mirror)):
        list2 = mirror[i][1]
        if len(list2) > length * 2:
            hd, c, position, f = naklist(length, list2)
            if hd:
                rem = set([1, 2, 3, 4, 5, 6, 7, 8, 9])
                for k in range(length):
                    rem.remove(mirror[i][0][position[k]])
                for k in range(length):
                    for m in rem:
                        p[boxl[i][c[k]]][m - 1] = 0
                message = ''
                if verbose > 1:
                    message = 'Hidden ' + ['single', 'pair', 'triple', 'quad'][length - 1] + ' in ' + \
                        ['box ', 'row ', 'column '][i // 9] + \
                        str(i % 9 + 1)
                if verbose > 2:
                    message = 'Hidden ' + ['single', 'pair', 'triple', 'quad'][length - 1] + ' in ' + [
                        'box ', 'row ', 'column '][i // 9] + str(
                        i % 9 + 1) + ': '
                    for m in range(length):
                        message += cell(boxl[i][c[m]])
                        if m < length - 1:
                            message += ' '
                return (s, p, message, True, False)
    return (s, p, '', False, False)


def naklist(length, list):
    """Naked pair, triple or quad."""
    if length == 2:
        np, c, position, f = nakpair(list)
        return (np, c, position, f)
    naked = []
    num = set()
    for i in list:
        if len(i) <= length:
            naked.append(i)
            for j in i:
                num.add(j)
    for c in itertools.combinations(num, length):
        n = 0
        for k in naked:
            if set(k).issubset(set(c)):
                n += 1
        if n == length:
            nak = []
            position = []
            m = 0
            f = []
            for k in list:
                if set(k).issubset(set(c)):
                    nak.append(k)
                    position.append(m)
                else:
                    if set(k).intersection(set(c)):
                        f.append(m)
                m += 1
            if len(f) > 0:
                return (True, c, position, f)
    return (False, 0, 0, 0)


def nakpair(list):
    """Naked pair."""
    pair = []
    for i in list:
        if len(i) == 2:
            pair.append(i)
    if len(set(pair)) < len(pair):
        pairs = []
        for i in range(len(pair)):
            for j in range(i + 1, len(pair)):
                if pair[i] == pair[j]:
                    pairs.append(pair[i])
        for pa in pairs:
            other = set(list) - set([pa])
            f = []
            for q in other:
                if pa[0] in q or pa[1] in q:
                    f.append(list.index(q))
            if len(f) > 0:
                position = [i for i, x in enumerate(list) if x == pa]
                return (True, pa, position, f)
    return (False, 0, 0, 0)



# -*- coding: utf-8 -*-
"""Modules for output."""


def output(s):
    """Convert to text output format."""
    out = '  1 2 3 4 5 6 7 8 9\n'
    out += ' +-----+-----+-----+\n'
    for i in range(9):
        out += str(i + 1) + '|'
        for j in range(9):
            v = s[i * 9 + j]
            if v == 0:
                out += ' '
            else:
                out = out + str(v)
            if j % 3 == 2:
                out += "|"
            else:
                out += ' '
        out += '\n'
        if i % 3 == 2:
            out += ' +-----+-----+-----+\n'
    return out


def url(s):
    """Convert to URL."""
    out = "http://www.sudoku-solutions.com/index.php?puzzle="
    for i in range(81):
        v = str(s[i])
        if v == 0:
            out = out + '.'
        else:
            out = out + str(v)
    return out


def short(s):
    """Convert to short form."""
    sudoku = ''
    for j in s:
        sudoku += str(j)
    return sudoku
